package com.jpattern.orm;

import java.util.List;

import com.jpattern.orm.exception.OrmConfigurationException;
import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.session.ISession;

/**
 * 
 * @author Francesco Cina
 *
 * 21/mag/2011
 */
public interface IJPOrm {
	
	/**
	 * Return a session from the current IJPOrm implementation
	 * @return
	 */
	ISession session() throws OrmException;
	
	/**
	 * Destroy the current IJPOrm instance and all it's references.
	 * After the execution of this method it will be not possible to call it through the
	 * static method JOPOrm.get(String jpOrmName) 
	 */
	void destory();
	
	/**
	 * Register a new class to be managed as a bean. 
	 * Information about the bean (primary keys) are loaded from the database.
	 * @param <T>
	 * @param clazz
	 * @throws OrmConfigurationException
	 */
	<T> void register(Class<T> clazz) throws OrmConfigurationException;
	
	/**
	 * Register a new class to be managed as a bean. 
	 * enableDBScan sets if whether load information about the bean (primary keys) from the database
	 * or use only the class annotations.
	 * @param <T>
	 * @param clazz
	 * @param enableDBScan
	 * @throws OrmConfigurationException
	 */
	<T> void register(Class<T> clazz, boolean enableDBScan) throws OrmConfigurationException;
	
	/**
	 * Register a list of classes to be managed as a beans. 
	 * enableDBScan sets if whether load information about the beans (primary keys) from the database
	 * or use only the class annotations.
	 * @param <T>
	 * @param clazz
	 * @param enableDBScan
	 * @throws OrmConfigurationException
	 */
	void register(List<Class<?>> classes, boolean enableDBScan) throws OrmConfigurationException;
}
