package com.jpattern.orm.classmapper;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.sql.DataSource;

import com.jpattern.logger.ILogger;
import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.logger.OrmLogger;

/**
 * 
 * @author Francesco Cina
 *
 * 13/giu/2011
 */
public class DBTableMapBuilder<T> implements ITableMapBuilder<T> {

	private final DataSource dataSource;
	private final ITableMap<T> tableMap;
	private final ILogger logger = OrmLogger.getOrmLogger(this.getClass());
	private final boolean enableDBScan;

	public DBTableMapBuilder(DataSource dataSource, ITableMap<T> tableMap, boolean enableDBScan) {
		this.dataSource = dataSource;
		this.tableMap = tableMap;
		this.enableDBScan = enableDBScan;
	}
	
	@Override
	public void updateTableMap() throws SQLException {
		if (enableDBScan) {
			Connection connection = null;
			try { 
				connection = dataSource.getConnection();
				mapField(connection, tableMap.getSchemaName(), tableMap.getTableName());
			} finally {
				if (connection != null) {
					try {
						connection.close();
					} catch (SQLException e) {
						throw new OrmException(e);
					}
				}
			}
		}
	}

	private void mapField(Connection connection, String schemaName, String tableName) throws SQLException {
		exploreMetaData(connection, schemaName, tableName);
	}
	
	private void exploreMetaData(Connection connection, String schemaName, String tableName) throws SQLException {
        DatabaseMetaData dbmd = null;
        dbmd = connection.getMetaData();
        String schema = null;
        if (schemaName!=null && schemaName.length()>0) {
        	schema = schemaName;
        }
        if (tableMap.getPrimaryKeyFieldNames().size()==0) {
    		logger.info("getPrimaryKeyColumns", "No annotations used to identify primary keys. Search them in the DB metadata.");
	    	ResultSet res;
	        res = dbmd.getPrimaryKeys(null, schema, tableName);
	        while (res.next()) {
	        	tableMap.getPrimaryKeyFieldNames().add( tableMap.getJavaFieldName( res.getString(Constant.COLUMN_NAME) ) );
	        }
	        res.close();
    	} else {
    		logger.info("getPrimaryKeyColumns", "Annotations are used to identify primary keys. DB metadata will not be used to retrieve primary keys.");
    	}
        logger.info("getPrimaryKeyColumns", "following field will be used as primary key for table " + schemaName + "." + tableName + ": " + tableMap.getPrimaryKeyFieldNames());
    }
	
}
